import time
import os
import requests
import subprocess
# 从selenium导入webdriver
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
import platform
from selenium.common.exceptions import NoSuchElementException
import traceback, sys
import argparse
import yaml


class ecnu_login:
    def __init__(self, driver_path=None, tmp_path=None):
        options = webdriver.ChromeOptions()

        options.add_argument("--no-sandbox")
        options.add_argument('--disable-dev-shm-usage')
        options.add_argument('--user-agent=Mozilla/5.0 HAHA')
        options.add_experimental_option("detach", True)
        options.add_argument('--headless')  # 浏览器隐式启动
        options.add_argument("--window-size=1920,1080")
        self.url = "https://login.ecnu.edu.cn/"
        self.driver = None
        if tmp_path:
            options.add_argument("crash-dumps-dir=" + tmp_path)
        self.options = options
        self.driver_path = driver_path

    def get_drive(self):
        print('driver is going to init!')
        self.driver = webdriver.Chrome(service=Service(self.driver_path) if self.driver_path else None,
                                       options=self.options)
        print('driver is inited!')
        self.driver.get(self.url)
        time.sleep(0.5)
        self.driver.refresh()
        time.sleep(2)

    #     todo:准备打包
    def refresh_drive(self):
        self.driver.get(self.url)
        self.driver.refresh()
        time.sleep(3)

    def close_driver(self):
        self.driver.close()
        self.driver.quit()

    def login_check(self, username, password):
        print('checking net......')
        if self.print_ip():
            return True
        else:
            return self.login(username, password)

    def print_ip(self):
        try:
            time.sleep(4)
            ipv4 = self.driver.find_element(By.ID, 'ipv4')
            if ipv4:
                print('当前IP：', ipv4.text)
                return True
        except Exception as e:
            return False

    def login(self, username, password):
        if self.print_ip():
            print('目前已登录！')
            return True
        driver = self.driver
        try:
            driver.refresh()
            time.sleep(2)
            username_ele = driver.find_element(By.ID, "username")
            password_ele = driver.find_element(By.ID, "password")
            username_ele.send_keys(username)
            password_ele.send_keys(password)
            print(rf'当前登录账户:{username}')
            # driver.find_element(By.ID, 'login-account').click()
            button = driver.find_element(By.ID, 'login-account')
            driver.execute_script("arguments[0].click();", button)
            time.sleep(2)
            try:
                ipv4 = driver.find_element(By.ID, 'ipv4')
                print('登录成功')
                print('当前IP：', ipv4.text)
                return True
            except NoSuchElementException as e:
                print(driver.find_element(By.XPATH,
                                          "/html/body/div[2]/div[@class='component dialog confirm active']/div[@class='content']/div[@class='section']")
                      .text)
                return False
        except Exception as e:
            print('登录失败！')
            traceback.print_exc()
            driver.save_screenshot('err.png')
            return False

    def logout(self):
        driver = self.driver
        try:
            driver.find_element(By.ID, 'logout').click()
            time.sleep(3)
            driver.find_element(By.CLASS_NAME, 'btn-confirm').click()
            print('登出成功')
            return True
        except Exception as e:
            print('登出失败！')
            traceback.print_exc()
            return False


def main():
    if platform.system().lower() == 'windows':
        driver_path = os.path.expanduser("~/chrome/chromedriver.exe")
    else:
        driver_path = os.path.expanduser("~/chrome/chromedriver")

    login = ecnu_login(driver_path=driver_path, tmp_path=os.path.expanduser("~/chrome/tmp"))
    login.get_drive()
    parser = argparse.ArgumentParser(
        prog='login',  # 程序名
        description='login or logout',  # 描述
        epilog='Copyright(r), 2023'  # 说明信息
    )

    parser.add_argument('-t', default='login_check')
    parser.add_argument('-u', default='')
    parser.add_argument('-p', default='')
    args = parser.parse_args()
    config_file = os.path.expanduser("~/.ecnu_login")
    if os.path.exists(config_file):
        opt = vars(args)
        yaml_args = yaml.load(open(config_file), Loader=yaml.FullLoader)
        if args.u != '' and 'u' in yaml_args:
            yaml_args.pop('u')
        if args.p != '' and 'p' in yaml_args:
            yaml_args.pop('p')
        opt.update(yaml_args)
    if args.t == 'login_check':
        login.login_check(args.u, args.p)
    elif args.t == 'login':
        login.login(args.u, args.p)
    elif args.t == 'logout':
        login.logout()



import sys
import numpy as np
import numpy.lib.format
import struct


class Add():
    def __init__(self):
        self.datas = None
        self.count = 0

    def add(self, np_data):
        if self.datas is None:
            self.datas = np_data.astype(np.float64)
        else:
            self.datas += np_data.astype(np.float64)
        self.count += 1

    def get_mean(self):
        return self.datas / self.count


def save(file, array):
    magic_string = b"\x93NUMPY\x01\x00v\x00"
    header = bytes(("{'descr': '" + array.dtype.descr[0][1] + "', 'fortran_order': False, 'shape': " + str(
        array.shape) + ", }").ljust(127 - len(magic_string)) + "\n", 'utf-8')
    if type(file) == str:
        file = open(file, "wb")
    file.write(magic_string)
    file.write(header)
    file.write(array.tobytes())


def pack(array):
    size = len(array.shape)
    return bytes(array.dtype.byteorder.replace('=', '<' if sys.byteorder == 'little' else '>') + array.dtype.kind,
                 'utf-8') + array.dtype.itemsize.to_bytes(1, byteorder='little') + struct.pack(f'<B{size}I', size,
                                                                                               *array.shape) + array.tobytes()


def load(file):
    if type(file) == str:
        file = open(file, "rb")
    header = file.read(128)
    if not header:
        return None
    descr = str(header[19:25], 'utf-8').replace("'", "").replace(" ", "")
    shape = tuple(int(num) for num in
                  str(header[60:120], 'utf-8').replace(', }', '')
                  .replace('(', '').replace(')', '').replace(' ', '').split(',') if num)
    datasize = numpy.lib.format.descr_to_dtype(descr).itemsize
    for dimension in shape:
        datasize *= dimension
    return np.ndarray(shape, dtype=descr, buffer=file.read(datasize))


def unpack(data):
    dtype = str(data[:2], 'utf-8')
    dtype += str(data[2])
    size = data[3]
    shape = struct.unpack_from(f'<{size}I', data, 4)
    datasize = data[2]
    for dimension in shape:
        datasize *= dimension
    return np.ndarray(shape, dtype=dtype, buffer=data[4 + size * 4:4 + size * 4 + datasize])


if __name__ == "__main__":
    import io
    from timeit import default_timer as timer
    from datetime import timedelta

    iterations = 100000
    testarray = np.random.rand(3, 64, 64).astype('float32')

    start = timer()
    for i in range(iterations):
        buffer = io.BytesIO()
        np.save(buffer, testarray)
        numpy_save_data = buffer.getvalue()
    print("numpy.save:", timedelta(seconds=timer() - start))

    start = timer()
    for i in range(iterations):
        buffer = io.BytesIO()
        save(buffer, testarray)
        fastnumpyio_save_data = buffer.getvalue()
    print("fastnumpyio.save:", timedelta(seconds=timer() - start))

    start = timer()
    for i in range(iterations):
        fastnumpyio_pack_data = pack(testarray)
    print("fastnumpyio.pack:", timedelta(seconds=timer() - start))

    start = timer()
    for i in range(iterations):
        buffer = io.BytesIO(numpy_save_data)
        test_numpy_save = np.load(buffer)
    print("numpy.load:", timedelta(seconds=timer() - start))

    start = timer()
    for i in range(iterations):
        buffer = io.BytesIO(fastnumpyio_save_data)
        test_fastnumpyio_save = load(buffer)
    print("fastnumpyio.load:", timedelta(seconds=timer() - start))

    start = timer()
    for i in range(iterations):
        test_fastnumpyio_pack = unpack(fastnumpyio_pack_data)
    print("fastnumpyio.unpack:", timedelta(seconds=timer() - start))

    print("numpy.save+numpy.load == fastnumpyio.save+fastnumpyio.load:",
          np.array_equal(test_numpy_save, test_fastnumpyio_save))
    print("numpy.save+numpy.load == fastnumpyio.pack+fastnumpyio.unpack:",
          np.array_equal(test_numpy_save, test_fastnumpyio_pack))



import os.path
import numpy as np
from einops import rearrange
from rasterio.transform import from_origin
import netCDF4 as nc
import rasterio
from rasterio.transform import from_gcps
from rasterio.control import GroundControlPoint as GCP
from typing import Tuple


class Coordinate:
    def __init__(self, left, top, x_res=None, y_res=None, right=None, bottom=None, h=None, w=None):
        self.left = left
        self.top = top
        self.x_res = x_res
        self.y_res = y_res
        self.h = h
        self.w = w
        self.right = right
        self.bottom = bottom
        if self.x_res is None:
            if self.right is None or self.w is None:
                raise Exception('x_res is None, right or w is also None')
            else:
                self.x_res = (self.right - self.left) / self.w
        if self.y_res is None:
            if self.bottom is None or self.h is None:
                raise Exception('y_res is None, bottom or h is also None')
            else:
                self.y_res = (self.top - self.bottom) / self.h
        if self.right is None:
            if self.x_res is None or self.w is None:
                raise Exception('right is None, x_res or w is also None')
            else:
                self.right = self.left + self.x_res * self.w
        if self.bottom is None:
            if self.y_res is None or self.h is None:
                raise Exception('bottom is None, y_res or h is also None')
            else:
                self.bottom = self.top - self.y_res * self.h
        if self.w is None:
            if self.right is None or self.x_res is None:
                raise Exception('w is None, right or x_res is also None')
            else:
                self.w = (self.right - self.left) / self.x_res
        if self.h is None:
            if self.bottom is None or self.y_res is None:
                raise Exception('h is None, bottom or y_res is also None')
            else:
                self.h = (self.top - self.bottom) / self.y_res
        self.h, self.w = int(self.h), int(self.w)
        self.ld = (self.left + self.right) / 2


def dms_to_d10(data):
    d, m, s = float(data[0]), float(data[1]), float(data[2])
    return d + 1 / 60 * m + 1 / 60 / 60 * s


def make_dms(s):
    print(s)
    temp = s.split('°')
    s_d = temp[0]
    temp = temp[1].split('′')
    s_m = temp[0]
    if len(temp) > 1 and temp[1]:
        s_s = temp[1].split('″')[0]
    else:
        s_s = 0
    result = [float(s_d), float(s_m), float(s_s)]
    return result


def _save_np2tif(np_data, output_dir, out_name, coordinate=None, resolution=None, dtype=None, print_log=False,
                 transform=None):
    h, w = np_data.shape
    os.makedirs(output_dir, exist_ok=True)
    save_path = os.path.join(output_dir, out_name)
    if coordinate:
        # 创建带地理坐标信息的 GeoTIFF 文件
        left, top = coordinate
        res1, res2 = resolution
        # 左上角坐标， 先经后纬
        transform = from_origin(left, top, res1, res2)
        with rasterio.open(save_path, "w", driver="GTiff", width=w, height=h, count=1,
                           dtype=dtype if dtype else np_data.dtype, crs="EPSG:4326", transform=transform) as dst:
            dst.write(np_data, 1)
        if print_log:
            print(f"GeoTIFF '{save_path}' generated with geographic coordinates.")
    elif transform:
        with rasterio.open(save_path, "w", driver="GTiff", width=w, height=h, count=1,
                           dtype=dtype if dtype else np_data.dtype, crs="EPSG:4326", transform=transform) as dst:
            dst.write(np_data, 1)
        if print_log:
            print(f"GeoTIFF '{save_path}' generated with geographic coordinates.")
    else:
        # 将数据写入TIFF文件
        with rasterio.open(save_path, "w", width=w, height=h, count=1, dtype=dtype if dtype else np_data.dtype) as dst:
            dst.write(np_data, 1)
        if print_log:
            print(f"TIFF image saved as '{save_path}'")


def np2tif(input_data, save_path='np2tif_dir', out_name='', left=None, top=None, x_res=None, y_res=None, dtype=None,
           dim_value=None, coord=None, print_log=True, transform=None):
    if type(input_data) == str:
        np_data = np.load(input_data)
        if np_data is None:
            print(f'load {input_data} is None')
    else:
        np_data = input_data
    shape = np_data.shape
    if len(shape) < 2:
        raise Exception(str(shape) + 'is less than 2 Dimensions')
    mode_list = ['d' + str(i) for i in range(len(shape))]

    mode_str = ' '.join(mode_list) + '->(' + ' '.join(mode_list[:-2]) + ') ' + mode_list[-2] + ' ' + mode_list[-1]
    np_data = rearrange(np_data, mode_str)
    if left is not None and top is not None:
        coordinate = (left, top)
    elif coord is not None:
        coordinate = (coord.left, coord.top)
        x_res, y_res = coord.x_res, coord.y_res
    else:
        coordinate = None
    for idx, single_np in enumerate(np_data):
        name = ''
        idx_tmp = idx
        for s in range(len(shape[:-2])):
            name += '-'
            temp = int(idx_tmp // np.prod(shape[s + 1:-2], axis=None))
            if dim_value is not None:
                plus_name = str(dim_value[s]['value'][temp])
            else:
                plus_name = str(temp)
            name += plus_name
            idx_tmp -= temp * np.prod(shape[s + 1:-2], axis=None)
        name = out_name + name + '.tif'
        # if name == '':
        #     name = out_name + '.tif'
        # else:
        #     name = out_name + '-' + name + '.tif'
        _save_np2tif(single_np, save_path, name, coordinate=coordinate, resolution=(x_res, y_res), dtype=dtype,
                     print_log=print_log, transform=transform)


def nc2tif(input_data, save_path='np2tif_dir', lock=None):
    np_data, dim_value = nc2np(input_data, lock)
    np2tif(np_data, save_path, dim_value=dim_value)


def nc2np(input_data, lock=None, return_dim=True):
    if type(input_data) == str:
        if lock:
            lock.acquire()
        nc_data = nc.Dataset(input_data)  # 读取.nc文件，传入f中。此时f包含了该.nc文件的全部信息
        if lock:
            lock.release()
    else:
        nc_data = input_data
    vars = []
    max_shape = 0
    dimensions = {}
    for name, var in nc_data.variables.items():
        if len(var.shape) > max_shape:
            max_shape = len(var.shape)
            vars = [name]
        elif len(var.shape) == max_shape:
            vars.append(name)
        if len(var.shape) == 1:
            dimensions[name] = list(nc_data[name][:])
    np_data = []
    for var in vars:
        np_data.append(np.array(nc_data[var][:]))
    np_data = np.array(np_data)
    value_idx = 0
    while 'value' + str(value_idx) in dimensions:
        value_idx += 1
    value_key = 'value' + str(value_idx)
    dimensions[value_key] = vars
    if return_dim:
        np_idx = [value_key] + list(nc_data[var].dimensions)
        dim_value = [{'dim_name': key, 'value': dimensions[key]} for key in np_idx]
    else:
        dim_value = None
    if type(input_data) == str:
        nc_data.close()
    return np_data, dim_value


def add_None(a, b):
    if a is None:
        return b
    else:
        return a + b


def get_transform_from_lonlat_matrices(
        lon_array: np.ndarray,
        lat_array: np.ndarray,
        gcp_density: int = 10,
        print_log=False,
        crs: str = "EPSG:4326"
) -> Tuple[rasterio.Affine, float]:
    """
    从每个像素的经纬度矩阵中，拟合并输出rasterio的transform（仿射变换矩阵）
    参数:
        lon_array: 2D numpy数组，shape为[height, width]，存放每个像素的经度
        lat_array: 2D numpy数组，shape为[height, width]，存放每个像素的纬度
        gcp_density: 控制点密度（每边提取的GCP数量），默认10（总GCP数≈10×10=100个）
                    范围越大，建议设越大（如20-50），拟合精度越高
        crs: 坐标系字符串（默认WGS84经纬度，EPSG:4326）

    返回:
        transform: rasterio.Affine对象，像素坐标到经纬度的仿射变换矩阵
        avg_error_km: 平均拟合误差（km），用于验证精度
    """
    # 1. 验证输入矩阵的有效性
    if lon_array.shape != lat_array.shape:
        raise ValueError(f"经度矩阵和纬度矩阵形状不匹配！lon_shape={lon_array.shape}, lat_shape={lat_array.shape}")
    height, width = lon_array.shape
    if height < 2 or width < 2:
        raise ValueError("矩阵尺寸过小（至少需要2×2像素），无法拟合transform")
    # 2. 均匀提取地面控制点（GCPs）- 避免边缘和密集采样，保证全局覆盖
    # 生成均匀分布的像素坐标（col, row）
    col_indices = np.linspace(0, width - 1, gcp_density, dtype=int)
    row_indices = np.linspace(0, height - 1, gcp_density, dtype=int)
    col_grid, row_grid = np.meshgrid(col_indices, row_indices)  # 网格状GCP分布
    # 3. 构造GCP列表（像素坐标 → 经纬度）
    gcps = []
    for row, col in zip(row_grid.flatten(), col_grid.flatten()):
        lon = lon_array[row, col]
        lat = lat_array[row, col]
        # 跳过无效经纬度（如NaN）
        if np.isnan(lon) or np.isnan(lat):
            continue
        # GCP格式：GCP(像素列, 像素行, 经度, 纬度)
        gcps.append(GCP(col, row, lon, lat))
    if len(gcps) < 3:
        raise ValueError(f"有效控制点不足3个（仅{len(gcps)}个），无法拟合仿射变换")
    # 4. 基于GCPs拟合transform（最小二乘法）
    transform = from_gcps(gcps)
    # 5. 计算拟合误差（验证精度）
    errors_km = []
    for gcp in gcps:
        # 用拟合的transform反推经纬度
        pred_lon, pred_lat = transform * (gcp.col, gcp.row)
        # 用半正矢公式计算实际经纬度与预测值的距离（km）
        error_km = haversine_distance(gcp.x, gcp.y, pred_lon, pred_lat)
        errors_km.append(error_km)
    avg_error_km = np.mean(errors_km)
    max_error_km = np.max(errors_km)
    if print_log:
        print(f"拟合完成：平均误差={avg_error_km:.3f}km，最大误差={max_error_km:.3f}km")
        print(f"提示：若误差过大（>0.5km），请增大gcp_density（当前={gcp_density}）")
    return transform, avg_error_km


def haversine_distance(lon1: float, lat1: float, lon2: float, lat2: float) -> float:
    """辅助函数：用半正矢公式计算两点间地球表面距离（km）"""
    lon1, lat1, lon2, lat2 = map(np.radians, [lon1, lat1, lon2, lat2])
    dlon = lon2 - lon1
    dlat = lat2 - lat1
    a = np.sin(dlat / 2) ** 2 + np.cos(lat1) * np.cos(lat2) * np.sin(dlon / 2) ** 2
    c = 2 * np.arcsin(np.sqrt(a))
    return 6371 * c  # 地球平均半径≈6371km


if __name__ == '__main__':
    np_data, dim = nc2np(r'C:\Users\jackSung\Desktop\download.nc')
    np2tif(np_data, 'com', dim_value=dim)
    print(dim)



import random

import torch
import torch.nn as nn
import torch.nn.functional as F
from einops import rearrange
import numpy as np


class ShiftConv2d0(nn.Module):
    def __init__(self, inp_channels, out_channels, stride):
        super(ShiftConv2d0, self).__init__()
        self.inp_channels = inp_channels
        self.out_channels = out_channels
        self.n_div = 5
        self.stride = stride
        g = inp_channels // self.n_div

        conv3x3 = nn.Conv2d(inp_channels, out_channels, 3, 1, 1)
        mask = nn.Parameter(torch.zeros((self.out_channels, self.inp_channels, 3, 3)), requires_grad=False)
        mask[:, 0 * g:1 * g, 1, 2] = 1.0
        mask[:, 1 * g:2 * g, 1, 0] = 1.0
        mask[:, 2 * g:3 * g, 2, 1] = 1.0
        mask[:, 3 * g:4 * g, 0, 1] = 1.0
        mask[:, 4 * g:, 1, 1] = 1.0
        self.w = conv3x3.weight
        self.b = conv3x3.bias
        self.m = mask

    def forward(self, x):
        y = F.conv2d(input=x, weight=self.w * self.m, bias=self.b, stride=self.stride, padding=1)
        return y


class ShiftConv2d1(nn.Module):
    def __init__(self, inp_channels, out_channels, stride):
        super(ShiftConv2d1, self).__init__()
        self.inp_channels = inp_channels
        self.out_channels = out_channels
        self.stride = stride
        self.weight = nn.Parameter(torch.zeros(inp_channels, 1, 3, 3), requires_grad=False)
        self.n_div = 5
        g = inp_channels // self.n_div

        channels_idx = list(range(inp_channels))
        random.shuffle(channels_idx)
        self.weight[channels_idx[0 * g:1 * g], 0, 1, 2] = 1.0  ## left
        self.weight[channels_idx[1 * g:2 * g], 0, 1, 0] = 1.0  ## right
        self.weight[channels_idx[2 * g:3 * g], 0, 2, 1] = 1.0  ## up
        self.weight[channels_idx[3 * g:4 * g], 0, 0, 1] = 1.0  ## down
        self.weight[channels_idx[4 * g:], 0, 1, 1] = 1.0  ## identity
        self.conv1x1 = nn.Conv2d(inp_channels, out_channels, 1)

    def forward(self, x):
        y = F.conv2d(input=x, weight=self.weight, bias=None, stride=self.stride, padding=1, groups=self.inp_channels)
        y = self.conv1x1(y)
        return y


class ShiftConv2d(nn.Module):
    def __init__(self, inp_channels, out_channels, conv_type='conv3', stride=1):
        super(ShiftConv2d, self).__init__()
        self.inp_channels = inp_channels
        self.out_channels = out_channels
        self.conv_type = conv_type
        if conv_type == 'low-training-memory':
            self.shift_conv = ShiftConv2d0(inp_channels, out_channels, stride=stride)
        elif conv_type == 'fast-training-speed':
            self.shift_conv = ShiftConv2d1(inp_channels, out_channels, stride=stride)
        elif conv_type == 'common':
            self.shift_conv = nn.Conv2d(inp_channels, out_channels, kernel_size=1, stride=stride)
        elif conv_type == 'conv3':
            self.shift_conv = nn.Conv2d(inp_channels, out_channels, kernel_size=3, stride=stride, padding=1)
        else:
            raise ValueError('invalid type of shift-conv2d')

    def forward(self, x):
        y = self.shift_conv(x)
        return y


class ACT(nn.Module):
    def __init__(self):
        super(ACT, self).__init__()
        # self.act = nn.Mish()
        self.act = nn.SiLU()

    def forward(self, x):
        return self.act(x)


class DownBlock(nn.Module):
    def __init__(self, c_lgan, downscale=2):
        super(DownBlock, self).__init__()
        self.down = nn.Conv2d(c_lgan, c_lgan * downscale, kernel_size=downscale, stride=downscale)
        self.norm = Norm(c_lgan * downscale)
        self.act = ACT()
        self.conv = nn.Conv2d(c_lgan * downscale, c_lgan * downscale, kernel_size=3, stride=1, padding=1)

    def forward(self, x):
        x = self.down(x)
        x = self.norm(x)
        x = self.act(x)
        x = self.conv(x)
        return x


class UpBlock(nn.Module):
    def __init__(self, c_lgan, downscale=2):
        super(UpBlock, self).__init__()
        self.conv = nn.Conv2d(c_lgan * downscale, c_lgan * downscale, kernel_size=3, stride=1, padding=1)
        self.norm = Norm(c_lgan * downscale)
        self.act = ACT()
        self.up = nn.ConvTranspose2d(c_lgan * downscale, c_lgan, kernel_size=downscale, stride=downscale)

    def forward(self, x):
        x = self.conv(x)
        x = self.norm(x)
        x = self.act(x)
        x = self.up(x)
        return x


class Norm(nn.Module):
    def __init__(self, c_in):
        super(Norm, self).__init__()
        self.norm = nn.BatchNorm2d(c_in)
        # self.norm = nn.GroupNorm(4, c_in, eps=1e-6, affine=True)

    def forward(self, x):
        x = self.norm(x)
        return x


class CubeEmbeding(nn.Module):
    def __init__(self, c_lgan, c_in, down_sample=1):
        super(CubeEmbeding, self).__init__()
        self.embeding = nn.Conv3d(c_in, c_lgan, kernel_size=(2, down_sample, down_sample),
                                  stride=(2, down_sample, down_sample), padding=(0, 0, 0))
        self.norm = Norm(c_lgan)
        self.act = ACT()
        self.conv2 = ShiftConv2d(c_lgan, c_lgan)

    def forward(self, x):
        x = self.embeding(x)
        x = self.norm(x[:, :, 0, :, :])
        x = self.act(x)
        x = self.conv2(x)
        return x


class CubeUnEmbeding(nn.Module):
    def __init__(self, c_lgan, c_in, downscale=2):
        super(CubeUnEmbeding, self).__init__()
        self.conv1 = nn.Conv2d(c_lgan, c_lgan, kernel_size=3, stride=1, padding=1)
        self.norm = Norm(c_lgan)
        self.act = ACT()
        self.up = nn.ConvTranspose2d(c_lgan, c_lgan, kernel_size=downscale, stride=downscale)
        self.conv2 = nn.Conv2d(c_lgan, c_in, kernel_size=3, stride=1, padding=1)

    def forward(self, x):
        x = self.conv1(x)
        x = self.norm(x)
        x = self.act(x)
        x = self.up(x)
        x = self.conv2(x)
        return x


class Head(nn.Module):
    def __init__(self, c_lgan, c_in, down_sample):
        super(Head, self).__init__()
        self.stage = nn.ModuleList()
        while down_sample >= 2:
            self.stage.append(
                ShiftConv2d(c_in, c_in * 2, stride=2))
            c_in *= 2
            self.stage.append(Norm(c_in))
            self.stage.append(ACT())
            down_sample = down_sample // 2

        self.conv2 = ShiftConv2d(c_in, c_lgan)

    def forward(self, x):
        # body
        for stage in self.stage:
            x = stage(x)
        x = self.conv2(x)
        return x


class Tail(nn.Module):
    def __init__(self, c_lgan, c_in, down_sample):
        super(Tail, self).__init__()
        self.down_sample = down_sample
        self.conv1 = nn.Conv2d(c_lgan, c_lgan, kernel_size=3, stride=1, padding=1)
        # self.norm = nn.BatchNorm2d(c_in * down_sample * down_sample)
        self.norm = Norm(c_lgan)
        self.act = ACT()
        if self.down_sample == 1:
            self.conv2 = nn.Conv2d(c_lgan, c_in, kernel_size=3,
                                   stride=1, padding=1)
        else:
            self.conv2 = nn.Conv2d(c_lgan, c_in * down_sample * down_sample, kernel_size=3,
                                   stride=1, padding=1)
            self.ps = nn.PixelShuffle(down_sample)

    def forward(self, x):
        x = self.conv1(x)
        x = self.norm(x)
        x = self.act(x)
        x = self.conv2(x)
        if self.down_sample != 1:
            x = self.ps(x)
        return x


class FD(nn.Module):
    def __init__(self, inp_channels, out_channels, exp_ratio=4):
        super(FD, self).__init__()
        # self.fc1 = MLP(inp_channels, inp_channels * exp_ratio)
        # self.fc2 = MLP(inp_channels * exp_ratio, out_channels)
        self.fc1 = ShiftConv2d(inp_channels, inp_channels * exp_ratio)
        self.fc2 = ShiftConv2d(inp_channels * exp_ratio, out_channels)
        # self.fc1 = nn.Conv2d(inp_channels, inp_channels * exp_ratio, kernel_size=3, stride=1, padding=1)
        # self.fc2 = nn.Conv2d(inp_channels * exp_ratio, out_channels, kernel_size=3, stride=1, padding=1)
        self.act1 = ACT()
        self.act2 = ACT()

    def forward(self, x):
        y = self.fc1(x)
        y = self.act1(y)
        y = self.fc2(y)
        return y


class LGAB(nn.Module):

    def __init__(self, channels, window_size=5, num_heads=8, split_part=3):
        super(LGAB, self).__init__()
        self.num_heads = num_heads
        self.window_size = window_size
        self.split_chns = [int(channels * 2 / split_part) for _ in range(split_part)]
        self.f_split_chns = [int(channels / split_part) for _ in range(split_part)]
        # self.project_inp = MLP(channels, channels * 3)
        # self.project_out = MLP(channels, channels)
        self.project_inp = ShiftConv2d(channels, channels * 2)
        self.f_project_inp = ShiftConv2d(channels, channels)
        self.project_out = ShiftConv2d(channels, channels)
        # self.f_project_inp = nn.Conv2d(channels, channels, kernel_size=1, stride=1)
        # self.project_inp = nn.Conv2d(channels, channels * 2, kernel_size=1, stride=1)
        # self.project_out = nn.Conv2d(channels, channels, kernel_size=1, stride=1)

        self.logit_scale = nn.Parameter(torch.log(10 * torch.ones((self.num_heads, 1, 1))), requires_grad=True)
        self.lr_logit_scale = nn.Parameter(torch.log(10 * torch.ones((self.num_heads, 1, 1))), requires_grad=True)
        # #########################################################################
        # mlp to generate continuous relative position bias
        self.cpb_mlp = nn.Sequential(nn.Linear(2, 512, bias=True),
                                     nn.ReLU(inplace=True),
                                     nn.Linear(512, self.num_heads, bias=False))

        # get relative_coords_table
        relative_coords_h = torch.arange(-(self.window_size - 1), self.window_size, dtype=torch.float32)
        relative_coords_w = torch.arange(-(self.window_size - 1), self.window_size, dtype=torch.float32)
        relative_coords_table = (torch.stack(
            torch.meshgrid([relative_coords_h, relative_coords_w], indexing='ij'))
                                 .permute(1, 2, 0).contiguous().unsqueeze(0))  # 1, 2*Wh-1, 2*Ww-1, 2

        relative_coords_table[:, :, :, 0] /= (self.window_size - 1)
        relative_coords_table[:, :, :, 1] /= (self.window_size - 1)
        relative_coords_table *= 8  # normalize to -8, 8
        relative_coords_table = torch.sign(relative_coords_table) * torch.log2(
            torch.abs(relative_coords_table) + 1.0) / np.log2(8)

        self.register_buffer("relative_coords_table", relative_coords_table)

        # get pair-wise relative position index for each token inside the window
        coords_h = torch.arange(self.window_size)
        coords_w = torch.arange(self.window_size)
        coords = torch.stack(torch.meshgrid([coords_h, coords_w], indexing='ij'))  # 2, Wh, Ww
        coords_flatten = torch.flatten(coords, 1)  # 2, Wh*Ww
        relative_coords = coords_flatten[:, :, None] - coords_flatten[:, None, :]  # 2, Wh*Ww, Wh*Ww
        relative_coords = relative_coords.permute(1, 2, 0).contiguous()  # Wh*Ww, Wh*Ww, 2
        relative_coords[:, :, 0] += self.window_size - 1  # shift to start from 0
        relative_coords[:, :, 1] += self.window_size - 1
        relative_coords[:, :, 0] *= 2 * self.window_size - 1
        relative_position_index = relative_coords.sum(-1)  # Wh*Ww, Wh*Ww
        self.register_buffer("relative_position_index", relative_position_index)
        # #########################################################################

    def wa(self, f, x, wsize):
        b, c, h, w = x.shape
        q = rearrange(
            f, 'b (head c) (h dh) (w dw) -> (b h w) head (dh dw) c',
            dh=wsize, dw=wsize, head=self.num_heads
        )
        k, v = rearrange(
            x, 'b (kv head c) (h dh) (w dw) -> kv (b h w) head (dh dw) c',
            kv=2, dh=wsize, dw=wsize, head=self.num_heads
        )
        atn = (F.normalize(q, dim=-1) @ F.normalize(k, dim=-1).transpose(-2, -1))
        t = torch.tensor(1. / 0.01).to(atn.device)
        logit_scale = torch.clamp(self.logit_scale, max=torch.log(t)).exp()
        atn = atn * logit_scale

        atn = atn.softmax(dim=-1)
        y_ = (atn @ v)
        y_ = rearrange(y_, '(b h w) head (dh dw) c-> b (head c) (h dh) (w dw)',
                       h=h // wsize, w=w // wsize, dh=wsize, dw=wsize, head=self.num_heads)
        return y_

    def forward(self, x):
        b, c, h, w = x.shape
        x_ = x
        x = self.project_inp(x_)
        xs = torch.split(x, self.split_chns, dim=1)
        f = self.f_project_inp(x_)
        fs = torch.split(f, self.f_split_chns, dim=1)
        wsize = self.window_size
        ys = []
        # window attention
        y_ = self.wa(fs[0], xs[0], wsize)
        ys.append(y_)
        # shifted window attention
        x_ = torch.roll(xs[1], shifts=(-wsize // 2, -wsize // 2), dims=(2, 3))
        f_ = torch.roll(fs[1], shifts=(-wsize // 2, -wsize // 2), dims=(2, 3))
        y_ = self.wa(f_, x_, wsize)
        y_ = torch.roll(y_, shifts=(wsize // 2, wsize // 2), dims=(2, 3))
        ys.append(y_)

        # long-range attentin
        # for longitude
        q = rearrange(fs[2], 'b (head c) h w -> (b h) head w c', head=self.num_heads)
        k, v = rearrange(xs[2], 'b (kv head c) h w -> kv (b h) head w c', kv=2, head=self.num_heads)
        atn = (F.normalize(q, dim=-1) @ F.normalize(k, dim=-1).transpose(-2, -1))
        t = torch.tensor(1. / 0.01).to(atn.device)
        logit_scale = torch.clamp(self.lr_logit_scale, max=torch.log(t)).exp()
        atn = atn * logit_scale
        # atn = (q @ k.transpose(-2, -1))
        atn = atn.softmax(dim=-1)
        v = (atn @ v)
        # for latitude
        q, k, v = (rearrange(q, '(b h) head w c -> (b w) head h c', h=h),
                   rearrange(k, '(b h) head w c -> (b w) head h c', h=h),
                   rearrange(v, '(b h) head w c -> (b w) head h c', h=h))
        atn = (F.normalize(q, dim=-1) @ F.normalize(k, dim=-1).transpose(-2, -1))
        atn = atn * logit_scale
        # atn = (q @ k.transpose(-2, -1))
        atn = atn.softmax(dim=-1)
        v = (atn @ v)
        y_ = rearrange(v, '(b w) head h c-> b (head c) h w', b=b)
        ys.append(y_)

        y = torch.cat(ys, dim=1)
        y = self.project_out(y)
        return y


class FEB(nn.Module):
    def __init__(self, inp_channels, exp_ratio=2, window_size=5, num_heads=8):
        super(FEB, self).__init__()
        self.exp_ratio = exp_ratio
        self.inp_channels = inp_channels
        self.down = DownBlock(inp_channels, downscale=2)
        self.up = UpBlock(inp_channels, downscale=2)

        self.FD = FD(inp_channels=inp_channels * 2, out_channels=inp_channels * 2, exp_ratio=exp_ratio)
        self.LGAB = LGAB(channels=inp_channels * 2, window_size=window_size, num_heads=num_heads)
        self.norm1 = Norm(inp_channels * 2)
        self.norm2 = Norm(inp_channels * 2)
        self.drop = nn.Dropout2d(0.2)

    def forward(self, x):
        res = x
        x = self.down(x)
        shortcut = x
        x = self.LGAB(x)
        x = self.drop(x)
        x = self.norm1(x) + shortcut
        shortcut = x
        x = self.FD(x)
        x = self.norm2(x) + shortcut
        x = self.up(x)
        x = x + res
        return x


if __name__ == '__main__':
    input_data = torch.zeros((5, 32, 32))
    conv = nn.Conv2d(5, 5, stride=2, kernel_size=3, padding=1)
    for i in range(5):
        input_data = conv(input_data)
        print(input_data.shape)



import os.path

import torch
import numpy as np
from einops import rearrange


class PredNormalization:
    def __init__(self, data_path):
        self.mean = np.load(os.path.join(data_path, 'mean_level.npy')).astype(np.float32)[2:]
        self.std = np.load(os.path.join(data_path, 'std_level.npy')).astype(np.float32)[2:]
        self.mean = torch.from_numpy(self.mean)
        self.std = torch.from_numpy(self.std)

    def norm(self, data):
        data = rearrange(data, 'b c h w->b h w c')
        data = (data - self.mean) / self.std
        return rearrange(data, 'b h w c->b c h w')

    def denorm(self, data):
        data = rearrange(data, 'b c h w->b h w c')
        data = data * self.std + self.mean
        return rearrange(data, 'b h w c->b c h w')


class PrecNormalization:
    def __init__(self, data_path):
        self.mean_qpe = torch.from_numpy(
            np.load(os.path.join(data_path, 'mean_level_qpe.npy')).astype(np.float32))
        self.std_qpe = torch.from_numpy(
            np.load(os.path.join(data_path, 'std_level_qpe.npy')).astype(np.float32))
        self.mean_fy = torch.from_numpy(
            np.load(os.path.join(data_path, 'mean_level_fy.npy')).astype(np.float32).mean(axis=0)[2:])
        self.std_fy = torch.from_numpy(
            np.load(os.path.join(data_path, 'std_level_fy.npy')).astype(np.float32).mean(axis=0)[2:])

    def norm(self, data, norm_type='fy'):
        if norm_type == 'fy':
            data = rearrange(data, 'b t c h w->b h w t c')
            data = (data - self.mean_fy) / self.std_fy
            return rearrange(data, 'b h w t c->b t c h w')
        elif norm_type == 'qpe':
            data = rearrange(data, 'b c h w->b h w c')
            data = (data - self.mean_qpe) / self.std_qpe
            return rearrange(data, 'b h w c->b c h w')

    def denorm(self, data, norm_type='fy'):
        if norm_type == 'fy':
            data = rearrange(data, 'b t c h w->b h w t c')
            data = data * self.std_fy + self.mean_fy
            return rearrange(data, 'b h w t c->b t c h w')
        elif norm_type == 'qpe':
            data = rearrange(data, 'b c h w->b h w c')
            data = data * self.std_qpe + self.mean_qpe
            return rearrange(data, 'b h w c->b c h w')


class PremNormalization:

    def __init__(self, prec_data_path):
        self.mean = torch.from_numpy(
            np.load(os.path.join(prec_data_path, 'imerg_mean.npy')).astype(np.float32))
        self.std = torch.from_numpy(
            np.load(os.path.join(prec_data_path, 'imerg_std.npy')).astype(np.float32))
        # self.mean = torch.Tensor(
        #     [3.6614e+03, 3.3748e+03, 3.3829e+03, 2.8368e+03, 2.5664e+03, 2.4914e+03, 2.4259e+03, 0.5723, 0, 0])
        # self.std = torch.Tensor([164.7376, 265.8857, 252.9820, 509.8994, 532.4901, 518.8191, 414.1427, 0.6308, 1, 1])
        # print(self.mean, self.std)

    def norm(self, data, fy_norm=True):
        data = rearrange(data, 'b c h w->b h w c')
        if fy_norm:
            data = (data - self.mean[:7]) / self.std[:7]
        else:
            # data[:, 0][data[:, 0] == 0] = torch.inf
            # data[:, 0] = 1 / torch.sqrt(data[:, 0].clone())
            data = (data - self.mean[7:]) / self.std[7:]
        return rearrange(data, 'b h w c->b c h w')

    def denorm(self, data, fy_norm=True):
        data = rearrange(data, 'b c h w->b h w c')
        if fy_norm:
            data = data * self.std[:7] + self.mean[:7]
        else:
            data = data * self.std[7:] + self.mean[7:]
            # data[:, 0][data[:, 0] == 0] = torch.inf
            # data[:, 0] = 1 / (data[:, 0].clone() ** 2)
        return rearrange(data, 'b h w c->b c h w')


class Normalization:
    def __init__(self, mean_std_npy, idx=None):
        mean_std = torch.from_numpy(mean_std_npy.astype(np.float32))
        if idx:
            self.mean = mean_std[0, idx[0]:idx[1]]
            self.std = mean_std[1, idx[0]:idx[1]]
        else:
            self.mean = mean_std[0]
            self.std = mean_std[1]

    def norm(self, data):
        data = rearrange(data, 'b c h w->b h w c')
        data = (data - self.mean) / self.std
        return rearrange(data, 'b h w c->b c h w')

    def denorm(self, data):
        data = rearrange(data, 'b c h w->b h w c')
        data = data * self.std + self.mean
        return rearrange(data, 'b h w c->b c h w')



import os
from openai import OpenAI
from tqdm import tqdm
from jacksung.utils.time import Stopwatch, get_time_str


def get_en_polish_prompt(text, prompt_type='polish'):
    polish_prompt = \
        fr'''
        # Rewrite the text in an academic writing style, using more appropriate vocabulary and sentence structure while keeping the original meaning unchanged:
        - Make sure the rewritten version conveys the same information and intention as the original text.
        - Please output the rewritten text directly in latex format, without including the original text, thinking logic, comments, explanations, etc.
        - Do not output any control commands that do not exist in the input content (such as \documentclass, \begin, \end, etc.), just use latex format to output mathematical formulas, symbols, references, or other latex instructions contained in the input content.
        - Note that special symbols and formulas are output in latex format, not directly output special characters.
        - Only when the input content contains control codes such as \par, the code needs to be added to the corresponding position of the output content.
        - If the input content only contains code and does not contain any substantial text content, the input content is directly output without any changes. Be careful not to miss symbols such as brackets.
        - Make sure that the output content can be compiled normally after replacing the input content in the original document.
        The following is the input content:
        {text}
        '''
    check_prompt = \
        fr'''
        # Correct grammatical errors and misspelled words in the input text while preserving the original meaning and format:
        - If the text is free of grammatical errors and spelling mistakes, output the original text without making any modifications
        - Ensure the corrected version maintains the exact same information and intent as the original text
        - Output only the corrected text directly without any additional content
        - Preserve all mathematical formulas, symbols, and special formatting exactly as input
        - Maintain the original document structure and formatting commands
        - Only modify actual grammatical errors and misspelled words
        - If the input contains LaTeX code, preserve it exactly and only correct text outside code blocks
        - Do not alter technical terms, proper nouns, or specialized vocabulary
        - If no errors are detected, output the original text unchanged
        - Do not include any explanations, comments, or thinking process in the output

        Input text:
        {text}
        '''
    if prompt_type == 'polish':
        return polish_prompt
    elif prompt_type == 'check':
        return check_prompt
    else:
        raise Exception(rf'Unknown prompt type {prompt_type}, please specify "polish" or "check"')


def get_cn_polish_prompt(text, prompt_type='polish'):
    polish_prompt = \
        fr'''
        # 用学术写作风格重写下面的文本,在保持原本涵义不变的情况下使用更合适的词汇和句子结构:
        - 确保改写后的版本传达的信息和意图与原文相同
        - 请直接以latex格式输出重写后的文本，不需要包含原文、思考逻辑、注释、解释说明等其他内容。
        - 不需要输出任何输入内容中不存在的控制命令（如\documentclass、\begin、\end等），只需要使用latex格式输出数学公式、符号、引用或者输入内容中所包含的其他latex指令。
        - 注意特殊符号和公式以latex格式输出，而不是直接输出特殊字符。
        - 仅在输入内容中包含\par类似的控制性代码时，在输出内容对应位置需要添加该代码。
        - 如果输入内容仅包含代码，不包含任何实质性的文本内容，则直接将输入内容不做任何改动输出。注意不要遗漏括号等符号。
        - 确保输出内容在原文档中替换输入内容后能够正常编译通过。
        - 重要：你需要确定输入内容所使用的语言，然后使用相同语言进行输出，以保证输入和输出语言一致。
        以下为输入内容:
        {text}
        '''
    check_prompt = \
        fr'''
        # 修正输入文本中的语法错误和单词拼写错误，同时保持原意和格式不变：\
        - 如果如数的文本没有语法错误和单词拼写则原文输出，不做任何改动
        - 确保修正后的文本与原文的信息和意图完全相同
        - 请直接输出修正后的文本，不需要包含原文、思考逻辑、注释、解释说明等其他内容
        - 不需要输出任何输入内容中不存在的控制命令（如\documentclass、\begin、\end等），只需要使用latex格式输出数学公式、符号、引用或者输入内容中所包含的其他latex指令
        - 注意特殊符号和公式以latex格式输出，而不是直接输出特殊字符
        - 仅在输入内容中包含\par类似的控制性代码时，在输出内容对应位置需要添加该代码
        - 如果输入内容仅包含代码，不包含任何实质性的文本内容，则直接将输入内容不做任何改动输出。注意不要遗漏括号等符号
        - 确保输出内容在原文档中替换输入内容后能够正常编译通过
        - 重要：你需要确定输入内容所使用的语言，然后使用相同语言进行输出，以保证输入和输出语言一致
        以下为输入内容:
        {text}
        '''
    if prompt_type == 'polish':
        return polish_prompt
    elif prompt_type == 'check':
        return check_prompt
    else:
        raise Exception(rf'Unknown prompt type {prompt_type}, please specify "polish" or "check"')


def merge_content(tex_dir, main_tex):
    result_tex = ''
    with open(os.path.join(tex_dir, main_tex), 'r', encoding='utf-8') as f:
        while True:
            line = f.readline()
            if not line:
                break
            if line.startswith(r'\input{') or line.startswith(r'\include{'):
                sub_tex_path = line.split('{')[1].split('}')[0]
                if not sub_tex_path.endswith('.tex'):
                    sub_tex_path += r'.tex'
                result_tex += merge_content(tex_dir, sub_tex_path) + '\n'
            else:
                result_tex += line
    return result_tex


class AI:
    def __init__(self, token, base_url, model_name='deepseek-r1:70b', prompt_type='polish'):
        self.client = OpenAI(api_key=token, base_url=base_url)
        self.model_name = model_name
        self.prompt_type = prompt_type

    def call_ai_polish(self, text, cn_prompt=False, prompt=None):
        response = self.client.chat.completions.create(
            model=self.model_name,
            messages=[
                {"role": "user",
                 "content": ((get_cn_polish_prompt(text, self.prompt_type) if cn_prompt else get_en_polish_prompt(
                     text, self.prompt_type)) if prompt is None else prompt.replace('{content}', text))}
            ],
            temperature=0.6,
            # max_tokens=1024,
            stream=False
        )
        # 逐步接收并处理响应
        # for chunk in response:
        #     print(chunk.choices[0].delta.content, end='')
        # print(response.choices[0].message.content)
        content = response.choices[0].message.content
        content = content.split('</think>')[1].strip().replace('\n\n', ' ')
        if text.startswith(r'\par ') and not content.startswith(r'\par '):
            print(rf'missing \par in polished text, append it to the beginning of the text.')
            content = r'\par ' + content
        if text.startswith(r'\par{') and not content.startswith(r'\par{'):
            print(rf'missing \par in polished text, append it to the beginning of the text.')
            content = r'\par{' + content
        if text.endswith(r'}') and not content.endswith(r'}'):
            print(r'missing } in polished text, append it to the beginning of the text.')
            content = content + r'}'
        return content


def polish(main_dir_path, tex_file, server_url, token='Your token here', model_name='deepseek-r1:70b', cn_prompt=False,
           prompt=None, rewrite_list=(r'\caption{', r'\par ', r'\par{'), skip_part_list=('figure', 'table', 'equation'),
           ignore_length=100, prompt_type='polish'):
    st = Stopwatch()
    ai = AI(token=token, base_url=server_url, model_name=model_name, prompt_type=prompt_type)
    result_tex = merge_content(main_dir_path, tex_file)
    new_tex = ''
    up_flag = False
    result_split = result_tex.split('\n')
    for idx, line in enumerate(result_split):
        spend_count = Stopwatch()
        line = line.strip()
        line_up_flag = True
        if line.startswith('%') or line.startswith('\\') or len(line) < ignore_length:
            for flag in rewrite_list:
                if line.startswith(flag):
                    line_up_flag = False
                    break
        else:
            line_up_flag = False

        for flag in skip_part_list:
            if line.count(r'\begin{' + flag) > 0:
                up_flag = True
                break
            if line.count(r'\end{' + flag) > 0:
                up_flag = False
                break

        if up_flag or line_up_flag:
            new_tex += line + '\n'
        else:
            try:
                print(rf'当前处理{idx}/{len(result_split)}行,总共用时{st.pinch()},当前时间:{get_time_str()}')
                print(f'Input[{line[:100]}{"..." if len(line) > 100 else line}]')
                polish_text = ai.call_ai_polish(line, cn_prompt, prompt)
                print(f'Polished:[{polish_text[:100]}{"..." if len(polish_text) > 100 else polish_text}]')
                print(rf'处理结束，耗时{spend_count.pinch()},共改写{len(line)}个字符')
                new_tex += polish_text + '\n'
            except Exception as e:
                tqdm.write(f'**e**{e}')
                new_tex += line + '\n'

    with open(rf'{main_dir_path}\old.tex', 'w', encoding='utf-8') as f:
        f.write(result_tex)
    with open(rf'{main_dir_path}\new.tex', 'w', encoding='utf-8') as f:
        f.write(new_tex)
    write_diff(main_dir_path)


def write_diff(dir_path):
    diff_tex = '''\RequirePackage{shellesc}
    \ShellEscape{pdfLatex new.tex} %编译新文档
    \ShellEscape{pdfLatex old.tex} %编译新文档
    \ShellEscape{latexdiff old.tex new.tex > diff_result.tex}
    \input{diff_result}
    \documentclass{dummy}'''
    with open(rf'{dir_path}\diff.tex', 'w', encoding='utf-8') as f:
        f.write(diff_tex)


if __name__ == "__main__":
    pass
